<?php
/**
 * This file is part of Avatar Privacy.
 *
 * Copyright 2019 Peter Putzer.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *  ***
 *
 * @package mundschenk-at/avatar-privacy
 * @license http://www.gnu.org/licenses/gpl-2.0.html
 */

namespace Avatar_Privacy\Tools\Images;

/**
 * A utility class providing some methods for dealing with SVG images.
 *
 * @since 2.3.0
 *
 * @author Peter Putzer <github@mundschenk.at>
 */
abstract class SVG {

	/**
	 * An array of allowed elements and attributes in wp_kses syntax.
	 *
	 * List initially compiled by Michael Pollett in Trac #24251.
	 *
	 * @link https://core.trac.wordpress.org/ticket/24251
	 *
	 * @var array
	 */
	const ALLOWED_ELEMENTS = [
		'a'              => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
			'href'              => true,
			'xlink:href'        => true,
			'xlink:title'       => true,
		],
		'circle'         => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'cx'                => true,
			'cy'                => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'r'                 => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
		],
		'clippath'       => [
			'class'         => true,
			'clippathunits' => true,
			'id'            => true,
		],
		'defs'           => true,
		'style'          => [
			'type' => true,
		],
		'desc'           => true,
		'ellipse'        => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'cx'                => true,
			'cy'                => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'rx'                => true,
			'ry'                => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => [],
		],
		'fegaussianblur' => [
			'class'                       => true,
			'color-interpolation-filters' => true,
			'id'                          => true,
			'requiredfeatures'            => true,
			'stddeviation'                => true,
		],
		'filter'         => [
			'class'                       => true,
			'color-interpolation-filters' => true,
			'filterres'                   => true,
			'filterunits'                 => true,
			'height'                      => true,
			'id'                          => true,
			'primitiveunits'              => true,
			'requiredfeatures'            => true,
			'width'                       => true,
			'x'                           => true,
			'xlink:href'                  => true,
			'y'                           => true,
		],
		'foreignobject'  => [
			'class'            => true,
			'font-size'        => true,
			'height'           => true,
			'id'               => true,
			'opacity'          => true,
			'requiredfeatures' => true,
			'style'            => true,
			'transform'        => true,
			'width'            => true,
			'x'                => true,
			'y'                => true,
		],
		'g'              => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'id'                => true,
			'display'           => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
			'font-family'       => true,
			'font-size'         => true,
			'font-style'        => true,
			'font-weight'       => true,
			'text-anchor'       => true,
		],
		'image'          => [
			'class'            => true,
			'clip-path'        => true,
			'clip-rule'        => true,
			'filter'           => true,
			'height'           => true,
			'id'               => true,
			'mask'             => true,
			'opacity'          => true,
			'requiredfeatures' => true,
			'style'            => true,
			'systemlanguage'   => true,
			'transform'        => true,
			'width'            => true,
			'x'                => true,
			'xlink:href'       => true,
			'xlink:title'      => true,
			'y'                => true,
		],
		'line'           => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'marker-end'        => true,
			'marker-mid'        => true,
			'marker-start'      => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
			'x1'                => true,
			'x2'                => true,
			'y1'                => true,
			'y2'                => true,
		],
		'lineargradient' => [
			'class'             => true,
			'id'                => true,
			'gradienttransform' => true,
			'gradientunits'     => true,
			'requiredfeatures'  => true,
			'spreadmethod'      => true,
			'systemlanguage'    => true,
			'x1'                => true,
			'x2'                => true,
			'xlink:href'        => true,
			'y1'                => true,
			'y2'                => true,
		],
		'marker'         => [
			'id'                  => true,
			'class'               => true,
			'markerheight'        => true,
			'markerunits'         => true,
			'markerwidth'         => true,
			'orient'              => true,
			'preserveaspectratio' => true,
			'refx'                => true,
			'refy'                => true,
			'systemlanguage'      => true,
			'viewbox'             => true,
		],
		'mask'           => [
			'class'            => true,
			'height'           => true,
			'id'               => true,
			'maskcontentunits' => true,
			'maskunits'        => true,
			'width'            => true,
			'x'                => true,
			'y'                => true,
		],
		'metadata'       => [
			'class' => true,
			'id'    => true,
		],
		'path'           => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'd'                 => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'marker-end'        => true,
			'marker-mid'        => true,
			'marker-start'      => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
		],
		'pattern'        => [
			'class'               => true,
			'height'              => true,
			'id'                  => true,
			'patterncontentunits' => true,
			'patterntransform'    => true,
			'patternunits'        => true,
			'requiredfeatures'    => true,
			'style'               => true,
			'systemlanguage'      => true,
			'viewbox'             => true,
			'width'               => true,
			'x'                   => true,
			'xlink:href'          => true,
			'y'                   => true,
		],
		'polygon'        => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'id'                => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'id'                => true,
			'class'             => true,
			'marker-end'        => true,
			'marker-mid'        => true,
			'marker-start'      => true,
			'mask'              => true,
			'opacity'           => true,
			'points'            => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
		],
		'polyline'       => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'id'                => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'marker-end'        => true,
			'marker-mid'        => true,
			'marker-start'      => true,
			'mask'              => true,
			'opacity'           => true,
			'points'            => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
		],
		'radialgradient' => [
			'class'             => true,
			'cx'                => true,
			'cy'                => true,
			'fx'                => true,
			'fy'                => true,
			'gradienttransform' => true,
			'gradientunits'     => true,
			'id'                => true,
			'r'                 => true,
			'requiredfeatures'  => true,
			'spreadmethod'      => true,
			'systemlanguage'    => true,
			'xlink:href'        => true,
		],
		'rect'           => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'height'            => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'rx'                => true,
			'ry'                => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'transform'         => true,
			'width'             => true,
			'x'                 => true,
			'y'                 => true,
		],
		'stop'           => [
			'class'            => true,
			'id'               => true,
			'offset'           => true,
			'requiredfeatures' => true,
			'stop-color'       => true,
			'stop-opacity'     => true,
			'style'            => true,
			'systemlanguage'   => true,
		],
		'svg'            => [
			'class'               => true,
			'clip-path'           => true,
			'clip-rule'           => true,
			'filter'              => true,
			'id'                  => true,
			'height'              => true,
			'mask'                => true,
			'preserveaspectratio' => true,
			'requiredfeatures'    => true,
			'style'               => true,
			'systemlanguage'      => true,
			'viewbox'             => true,
			'width'               => true,
			'x'                   => true,
			'xmlns'               => true,
			'xmlns:se'            => true,
			'xmlns:xlink'         => true,
			'y'                   => true,
		],
		'switch'         => [
			'class'            => true,
			'id'               => true,
			'requiredfeatures' => true,
			'systemlanguage'   => true,
		],
		'symbol'         => [
			'class'               => true,
			'fill'                => true,
			'fill-opacity'        => true,
			'fill-rule'           => true,
			'filter'              => true,
			'font-family'         => true,
			'font-size'           => true,
			'font-style'          => true,
			'font-weight'         => true,
			'id'                  => true,
			'opacity'             => true,
			'preserveaspectratio' => true,
			'requiredfeatures'    => true,
			'stroke'              => true,
			'stroke-dasharray'    => true,
			'stroke-dashoffset'   => true,
			'stroke-linecap'      => true,
			'stroke-linejoin'     => true,
			'stroke-miterlimit'   => true,
			'stroke-opacity'      => true,
			'stroke-width'        => true,
			'style'               => true,
			'systemlanguage'      => true,
			'transform'           => true,
			'viewbox'             => true,
		],
		'text'           => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'font-family'       => true,
			'font-size'         => true,
			'font-style'        => true,
			'font-weight'       => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'text-anchor'       => true,
			'transform'         => true,
			'x'                 => true,
			'xml:space'         => true,
			'y'                 => true,
		],
		'textpath'       => [
			'class'            => true,
			'id'               => true,
			'method'           => true,
			'requiredfeatures' => true,
			'spacing'          => true,
			'startoffset'      => true,
			'style'            => true,
			'systemlanguage'   => true,
			'transform'        => true,
			'xlink:href'       => true,
		],
		'title'          => true,
		'tspan'          => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'dx'                => true,
			'dy'                => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'font-family'       => true,
			'font-size'         => true,
			'font-style'        => true,
			'font-weight'       => true,
			'id'                => true,
			'mask'              => true,
			'opacity'           => true,
			'requiredfeatures'  => true,
			'rotate'            => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'systemlanguage'    => true,
			'text-anchor'       => true,
			'textlength'        => true,
			'transform'         => true,
			'x'                 => true,
			'xml:space'         => true,
			'y'                 => true,
		],
		'use'            => [
			'class'             => true,
			'clip-path'         => true,
			'clip-rule'         => true,
			'fill'              => true,
			'fill-opacity'      => true,
			'fill-rule'         => true,
			'filter'            => true,
			'height'            => true,
			'id'                => true,
			'mask'              => true,
			'stroke'            => true,
			'stroke-dasharray'  => true,
			'stroke-dashoffset' => true,
			'stroke-linecap'    => true,
			'stroke-linejoin'   => true,
			'stroke-miterlimit' => true,
			'stroke-opacity'    => true,
			'stroke-width'      => true,
			'style'             => true,
			'transform'         => true,
			'width'             => true,
			'x'                 => true,
			'xlink:href'        => true,
			'y'                 => true,
		],
	];
}
